#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime
from pathlib import Path
from typing import Callable, Dict, List, Optional


def diff_lines(
        lines_1: List[str],
        lines_2: List[str],
        filter_func: Callable[[str, int], bool] = None
) -> List[str]:
    len_1 = len(lines_1)
    len_2 = len(lines_2)
    len_min = min(len_1, len_2)
    diff = []
    for i in range(len_min):
        a = lines_1[i]
        b = lines_2[i]
        if filter_func and \
                filter_func(a, i) and \
                filter_func(b, i):
            continue
        if a != b:
            diff.append(f"{a.strip()} → {b.strip()}")
    if len_1 > len_min:
        for i in range(len_min, len_1):
            diff.append(lines_1[i])
    if len_2 > len_min:
        for i in range(len_min, len_2):
            diff.append(lines_2[i])
    return diff


def diff_text(
        text_1: str,
        text_2: str,
        filter_func: Callable[[str, int], bool] = None
) -> List[str]:
    lines_1 = text_1.splitlines(keepends=True)
    lines_2 = text_2.splitlines(keepends=True)
    return diff_lines(lines_1, lines_2, filter_func)


def interpolate_text(
        text: str,
        expressions: Dict[str, str]
) -> str:
    for k, v in expressions.items():
        text = text.replace(k, v)
    return text


def timestamp() -> str:
    return datetime.utcnow().isoformat()


def write_text_file(
        text: str,
        path: Path,
        diff_show_len: int = 5,
        diff_func: Callable[[str, str, Optional[Callable[[str, int], bool]]], List[str]] = None,
        filter_func: Callable[[str, int], bool] = None,
        log_func: Callable[[str], None] = None
) -> None:
    if not path.parent.exists():
        path.parent.mkdir(parents=True, exist_ok=True)
    if diff_func and path.exists():
        orig_text = path.read_text()
        diff = diff_func(orig_text, text, filter_func)
        diff_len = len(diff)
        if diff_len == 0:
            if log_func:
                log_func(f"No relevant changes found, skipping {path.name}.")
            return
        if log_func:
            log_func(f"Found {diff_len} change(s) to {path.name}.")
            for i in range(min(diff_len, diff_show_len)):
                log_func(f".. {diff[i]}")
    path.write_text(text)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import List

from scriptgen import BlockBuilder, IndentType, StringBuilder, timestamp


class CSharpBlockBuilder(BlockBuilder):

    default_semicolon_end: bool = False

    def __init__(
            self,
            header: str = "",
            semicolon_end: bool = None,
            indent_len: int = None,
            indent_space_len: int = None,
            indent_type: IndentType = None,
            rstrip: bool = None
    ) -> None:
        semicolon_end = semicolon_end if semicolon_end is not None else CSharpBlockBuilder.default_semicolon_end
        BlockBuilder.__init__(
            self,
            indent_len=indent_len,
            indent_space_len=indent_space_len,
            indent_type=indent_type,
            rstrip=rstrip
        )
        self.header = f"{header}{self.new_line}{{{self.new_line}" if header else f"{{{self.new_line}"
        self.footer = f"}}{';' if semicolon_end else ''}{self.new_line}"


def csharp_autogen() -> StringBuilder:
    return csharp_comment(f"Auto-generated: {timestamp()}")


def csharp_block() -> StringBuilder:
    return CSharpBlockBuilder()


def csharp_class(
        class_name: str,
        access_modifier: str = "public",
        base_class_name: str = None,
        interface_names: List[str] = None
) -> StringBuilder:
    interface_names = interface_names if interface_names is not None else []
    inheritance = []
    if base_class_name:
        inheritance.append(base_class_name)
    if interface_names:
        inheritance.extend(interface_names)
    inheritance_str = f" : {', '.join(inheritance)}" if inheritance else ""
    header = f"{access_modifier} class {class_name}{inheritance_str}"
    csb = CSharpBlockBuilder(
        header=header
    )
    return csb


def csharp_comment(*args: str) -> StringBuilder:
    sb = StringBuilder()
    for arg in args:
        sb.wl(f"// {arg}")
    return sb


def csharp_constructor(
        class_name: str,
        access_modifier: str = "public",
        parameters: List[str] = None
) -> StringBuilder:
    parameters_str = csharp_parameters(parameters)
    header = f"{access_modifier} {class_name}({parameters_str})"
    csb = CSharpBlockBuilder(
        header=header
    )
    return csb


def csharp_doc(*args: str) -> StringBuilder:
    sb = StringBuilder()
    sb.wl("/// <summary>")
    for arg in args:
        sb.wl(f"/// {sb.indent}{arg}")
    sb.wl("/// </summary>")
    return sb


def csharp_for_loop(
        var_name: str = "i",
        initial_val: str = "0",
        condition_max: str = None,
        condition_stmt: str = None,
        step_stmt: str = None,
        var_type: str = None
) -> StringBuilder:
    var_type = var_type if var_type else "var"
    condition_stmt = condition_stmt if condition_stmt else f"{var_name} < {condition_max}"
    step_stmt = step_stmt if step_stmt else f"++{var_name}"
    csb = CSharpBlockBuilder(
        header=f"for ({var_type} {var_name} = {initial_val}; {condition_stmt}; {step_stmt})"
    )
    return csb


def csharp_foreach_loop(
        enumerable_name: str,
        var_name: str = "v",
        var_type: str = None
) -> StringBuilder:
    var_type = var_type if var_type else "var"
    csb = CSharpBlockBuilder(
        header=f"foreach ({var_type} {var_name} in {enumerable_name})"
    )
    return csb


def csharp_method(
        method_name: str,
        access_modifier: str = "public",
        return_type: str = "void",
        parameters: List[str] = None
) -> StringBuilder:
    parameters_str = csharp_parameters(parameters)
    header = f"{access_modifier} {return_type} {method_name}({parameters_str})"
    csb = CSharpBlockBuilder(
        header=header
    )
    return csb


def csharp_namespace(namespace_name: str) -> StringBuilder:
    csb = CSharpBlockBuilder(
        header=f"namespace {namespace_name}"
    )
    return csb


def csharp_parameters(parameters: List[str]) -> StringBuilder:
    parameters = parameters if parameters else []
    parameters_len = len(parameters)
    sb = StringBuilder()
    if parameters_len > 1:
        sb.nl()
        for i, p in enumerate(parameters):
            sb.wt(p, addtl_indent_len=1)
            sb.wl("," if i < parameters_len - 1 else "")
    elif parameters_len == 1:
        p = parameters[0]
        sb.wt(p)
    return sb


def csharp_region(region_name: str) -> StringBuilder:
    bb = BlockBuilder(
        indent_len=0,
        header=f"#region {region_name}{StringBuilder.new_line * 2}",
        footer=f"{StringBuilder.new_line}#endregion {region_name}{StringBuilder.new_line}",
    )
    return bb


def csharp_try_catch(
        catch_var_name: str = "Exception ex"
) -> (StringBuilder, StringBuilder, StringBuilder):
    sb = StringBuilder()
    csb_try = CSharpBlockBuilder("try")
    sb.wb(csb_try)
    csb_catch = CSharpBlockBuilder(f"catch ({catch_var_name})")
    sb.wb(csb_catch)
    return sb, csb_try, csb_catch


def csharp_usings(
        *args: str,
        **kwargs: str
) -> StringBuilder:
    sb = StringBuilder()
    for arg in args:
        sb.wl(f"using {arg};")
    for k, v in kwargs.items():
        sb.wl(f"using {k} = {v};")
    return sb



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .core import \
    IndentType, \
    StringBuilder, \
    BlockBuilder, \
    diff_lines, \
    diff_text, \
    interpolate_text, \
    timestamp, \
    write_text_file



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from scriptgen import BlockBuilder, StringBuilder, timestamp


def markdown_autogen() -> StringBuilder:
    return markdown_comment(f"Auto-generated: {timestamp()}", padded=False)


def markdown_code_block(language: str = None) -> StringBuilder:
    language = language if language else ""
    bb = BlockBuilder(
        indent_len=0,
        header=f"```{language}{StringBuilder.new_line}",
        footer=f"```{StringBuilder.new_line}"
    )
    return bb


def markdown_comment(
        *args: str,
        padded: bool = True) -> StringBuilder:
    sb = StringBuilder()
    if args:
        if padded:
            sb.nl()
        for arg in args:
            sb.wl(f"[//]: # ({arg})")
        if padded:
            sb.nl()
    return sb


def markdown_inline(
        value: str,
        decorator: str = None) -> str:
    decorator = decorator if decorator else ""
    return f"{decorator}{value}{decorator[::-1]}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path
from setuptools import find_packages, setup

NAME = 'scriptgen'
VERSION = '0.0.3'
DESCRIPTION = 'A collection of script generation helpers and templates.'
HOME = Path(__file__).parent
README = (HOME / 'README.md').read_text()
CHANGELOG = (HOME / 'CHANGELOG.md').read_text()
AUTHOR = 'Elmer Nocon, fopoon'
AUTHOR_EMAIL = 'elmernocon@gmail.com'
LICENSE = 'MIT'
PLATFORMS = 'Any'
URL = 'https://github.com/Fopoon/scriptgen'
DOWNLOAD_URL = 'https://pypi.org/project/scriptgen/'
CLASSIFIERS = [
    'Development Status :: 1 - Planning',
    'License :: OSI Approved :: MIT License',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows :: Windows 10',
    'Operating System :: Unix',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3 :: Only',
]

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description='\n\n'.join([README, CHANGELOG]),
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    platforms=PLATFORMS,
    url=URL,
    download_url=DOWNLOAD_URL,
    classifiers=CLASSIFIERS,

    packages=find_packages(
        exclude=[
            "tests",
            "tests.*",
            "*.tests.*",
            "*.tests"
        ]
    ),
)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import annotations
from enum import Enum, auto
from os import linesep
from typing import List, Union


class IndentType(Enum):
    SPACES = auto()
    TABS = auto()


class StringBuilder:

    indent_tab: str = "\t"
    new_line: str = linesep

    default_indent_len: int = 0
    default_indent_space_len: int = 4
    default_indent_type: IndentType = IndentType.SPACES
    default_rstrip: bool = False

    def __init__(
            self,
            indent_len: int = None,
            indent_space_len: int = None,
            indent_type: IndentType = None,
            rstrip: bool = None
    ) -> None:
        indent_len = indent_len if indent_len is not None else StringBuilder.default_indent_len
        indent_space_len = indent_space_len if indent_space_len is not None else StringBuilder.default_indent_space_len
        indent_type = indent_type if indent_type is not None else StringBuilder.default_indent_type
        rstrip = rstrip if rstrip is not None else StringBuilder.default_rstrip
        self.indent_len = indent_len
        self.indent_space_len: int = indent_space_len
        self.indent_type: IndentType = indent_type
        self.rstrip: bool = rstrip
        self.strings: List[Union[str, StringBuilder]] = []

    def __str__(self) -> str:
        return self.build()

    @property
    def indent(self) -> str:
        return self._indent

    @property
    def indent_space(self) -> str:
        return self._indent_space

    @property
    def indent_space_len(self) -> int:
        return self._indent_space_len

    @indent_space_len.setter
    def indent_space_len(
            self,
            value: int
    ) -> None:
        self._indent_space_len = value
        self._indent_space = " " * value

    @property
    def indent_type(self) -> IndentType:
        return self._indent_type

    @indent_type.setter
    def indent_type(
            self,
            value: IndentType
    ) -> None:
        self._indent_type = value
        if IndentType.SPACES == value:
            self._indent = self.indent_space
        elif IndentType.TABS == value:
            self._indent = self.indent_tab

    def build(
            self,
            rstrip: bool = None
    ) -> str:
        if self.strings:
            rstrip = self.rstrip if rstrip is None else rstrip
            indent = self.indent * self.indent_len
            strings = [str(string) for string in self.strings]
            lines = "".join(strings)
            text = "".join([f"{indent}{line}" for line in lines.splitlines(keepends=True)])
            if rstrip:
                text = text.rstrip()
            return text
        return ""

    def clear(self) -> None:
        self.strings.clear()

    def nl(self) -> None:
        self.strings.append(self.new_line)

    def wb(
            self,
            string_builder: StringBuilder,
            rstrip: bool = None
    ) -> None:
        rstrip = self.rstrip if rstrip is None else rstrip
        string_builder.rstrip = rstrip
        self.strings.append(string_builder)

    def wt(
            self,
            text: str,
            addtl_indent_len: int = 0
    ) -> None:
        self.strings.append(f"{self.indent * addtl_indent_len}{text}")

    def wts(
            self,
            texts: List[str],
            addtl_indent_len: int = 0
    ) -> None:
        for text in texts:
            self.wt(text, addtl_indent_len)

    def wl(
            self,
            line: str,
            addtl_indent_len: int = 0
    ) -> None:
        self.strings.append(f"{self.indent * addtl_indent_len}{line}{self.new_line}")

    def wls(
            self,
            lines: List[str],
            addtl_indent_len: int = 0
    ) -> None:
        for line in lines:
            self.wl(line, addtl_indent_len)


class BlockBuilder(StringBuilder):

    default_indent_len: int = 1

    def __init__(
            self,
            header: str = "",
            footer: str = "",
            indent_len: int = None,
            indent_space_len: int = None,
            indent_type: IndentType = None,
            rstrip: bool = None
    ) -> None:
        indent_len = indent_len if indent_len is not None else BlockBuilder.default_indent_len
        StringBuilder.__init__(
            self,
            indent_len=indent_len,
            indent_space_len=indent_space_len,
            indent_type=indent_type,
            rstrip=rstrip
        )
        self.header: str = header
        self.footer: str = footer

    def build(
            self,
            rstrip: bool = None
    ) -> str:
        rstrip = self.rstrip if rstrip is None else rstrip
        contents = StringBuilder.build(self, rstrip)
        return "".join([self.header, contents, self.footer])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ._csharp import \
    CSharpBlockBuilder, \
    csharp_autogen, \
    csharp_block, \
    csharp_class, \
    csharp_comment, \
    csharp_constructor, \
    csharp_doc, \
    csharp_for_loop, \
    csharp_foreach_loop, \
    csharp_method, \
    csharp_namespace, \
    csharp_parameters, \
    csharp_region, \
    csharp_try_catch, \
    csharp_usings
